local addonName, addon = ...

local LCI = LibStub("LibCraftInfo-1.0")

local craftsDB = {}
addon.craftsDB = craftsDB

local myRealm = GetRealmName()
local myFaction = UnitFactionGroup("player")
local myName = UnitName("player")
local updatedBags = {}
local recipeColorCache = {}
local addonLoaded = {}
local options, newCraftMsgShown, bankOpen, lastRecipe

local ERR_SKILL_GAINED_S	= ERR_SKILL_GAINED_S:format("(.+)")		-- "You have gained the (.+) skill."
local ERR_LEARN_RECIPE_S	= ERR_LEARN_RECIPE_S:format("(.+)%")	-- "You have learned how to create a new item: (.+)%."
local ERR_SPELL_UNLEARNED_S	= ERR_SPELL_UNLEARNED_S:format("(.+)%")	-- "You have unlearned (.+)%."
local ERR_SKILL_UP_SI		= format("^%s$", ERR_SKILL_UP_SI:gsub("%%[sd]", "(%0+)"):gsub("%%s", "."))			-- "^Your skill in (.+) has increased to (%d+).$"
local ITEM_MIN_SKILL		= format("^%s$", ITEM_MIN_SKILL:gsub("%%s", ".+"):gsub("%(%%d%)", "%%((%%d+)%%)"))	-- "^Requires .+ %((.+)%)$"
local ITEM_REQ_SKILL		= format("^%s$", ITEM_REQ_SKILL:format("(.+)"))										-- "^Requires (.+)$"

-- used for checking item type and subtype returns from GetItemInfo
local AH_RECIPE_INDEX = 7
local RECIPE = select(AH_RECIPE_INDEX, GetAuctionItemClasses())
local BOOK = GetAuctionItemSubClasses(AH_RECIPE_INDEX)

addon.menu = {}

-- "apprentice" rank spell IDs, used for localization
addon.skills = {
	2018, -- Blacksmithing
	2108, -- Leatherworking
	2259, -- Alchemy
	2550, -- Cooking
	-- 2575, -- Mining
	3273, -- First Aid
	3908, -- Tailoring
	4036, -- Engineering
	7411, -- Enchanting
	25229, -- Jewelcrafting
	45357, -- Inscription
}
local skills = addon.skills

-- this lets us lookup trade skill spell IDs by name, eg:
-- skillsByName["Blacksmithing"] = 2018
-- used for getting trade skill ID from recipes and the trade skill frame
local skillsByName = {}

for i = 1, #skills do
	local spellID = skills[i]
	local spellName = GetSpellInfo(spellID)
	skillsByName[spellName] = spellID
	skills[spellID] = spellName
	skills[i] = nil
end

local specs = {
	[20219] = 4036, -- Gnomish Engineer
	[20222] = 4036, -- Goblin Engineer
	-- [28677] = 2259, -- Elixir Master
}

local specsByName = {}

for spellID in pairs(specs) do
	specsByName[GetSpellInfo(spellID)] = spellID
end


local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function(self, event, ...)
	addon[event](addon, ...)
end)
addon.frame = frame
function addon:RegisterEvent(event)
	frame:RegisterEvent(event)
end
function addon:UnregisterEvent(event)
	frame:UnregisterEvent(event)
end
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("UNIT_NAME_UPDATE")
addon:RegisterEvent("TRADE_SKILL_UPDATE")
addon:RegisterEvent("TRADE_SKILL_SHOW")
addon:RegisterEvent("LEARNED_SPELL_IN_TAB")
addon:RegisterEvent("BANKFRAME_OPENED")
addon:RegisterEvent("BANKFRAME_CLOSED")
addon:RegisterEvent("CHAT_MSG_SKILL")
addon:RegisterEvent("CHAT_MSG_SYSTEM")


local tooltip = CreateFrame("GameTooltip", "BoCTooltip", GameTooltip, "ShoppingTooltipTemplate")
-- _G[self:GetName().."TextRight1"]:SetFontObject("GameFontNormalSmall")
BoCTooltipTextLeft2:SetFontObject("GameFontNormalSmall")
tooltip:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
tooltip:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
local addLine = tooltip.AddLine
function tooltip:AddLine(text, color)
	if color then
		addLine(self, text, color.r, color.g, color.b)
	else
		addLine(self, text, 1.0, 1.0, 1.0)
	end
end


function addon:ChatMsg(msg)
	DEFAULT_CHAT_FRAME:AddMessage("[Book of Crafts] "..msg, 0.0, 1.0, 0.0)
end


SLASH_BookOfCrafts1 = "/boc"
SLASH_BookOfCrafts2 = "/bookofcrafts"
SlashCmdList.BookOfCrafts = function(cmd)
	InterfaceOptionsFrame_OpenToCategory(frame)
end


--[[

DB = {
	["Char|Realm"] = {
		skills = {
			spellID = {
				crafts = {
					-- any learnt trade skill crafts
					[craftID] = true,
					[...]
				},
				-- any learnt specialisation of the trade skill such as gnomish engineering, etc
				spec = spellID,
				link = "link",
				skillLevel = tradeSkillLevel,
			},
		},
		-- recipes this character possess
		recipes = {
			bags = {
				[itemID] = amount,
			},
			bank = {
				[itemID] = amount,
			},
		},
		faction = "Faction",
	},
}

]]

local defaults = {
	options = {
		sameFaction = true,
		showSkillLevel = true,
		showPlayer = true,
		debug = false,
		colorRecipes = {
			bags = true,
			bank = true,
			vendor = true,
			auction = true,
			guildBank = true,
		},
		colors = {
			usable   = {r = 0.0, g = 1.0, b = 0.0},
			lowSkill = {r = 1.0, g = 0.0, b = 0.0},
			noSpec   = {r = 1.0, g = 1.0, b = 0.0},
			knownBy  = {r = 0.5, g = 0.5, b = 1.0},
			useless  = {r = 0.5, g = 0.5, b = 0.5},
		},
	},
	data = {},
}

local function copyDefaults(src, dst)
	if not src then return {} end
	if not dst then dst = {} end
	for k, v in pairs(src) do
		if type(v) == "table" then
			dst[k] = copyDefaults(v, dst[k])
		elseif type(v) ~= type(dst[k]) then
			dst[k] = v
		end
	end
	return dst
end

function addon:ADDON_LOADED(addon)
	if self:HookAddon(addon) then
		return
	end
	
	if addon == addonName then
		local db = BookofCraftsDB or {}
		copyDefaults(defaults, db)
		
		BookofCraftsDB = db
		self.db = db
		options = db.options

		self.realm = db.data[myRealm] or {}
		db.data[myRealm] = self.realm
		
		self.player = self.realm[myName] or {
			skills = {},
			recipes = {
				bags = {},
				bank = {},
			},
			faction = myFaction,
		}
		self.realm[myName] = self.player
		
		for i, v in ipairs(self.options) do
			v:SetChecked(options[v.setting])
		end
		
		for i, swatch in pairs(self.swatches) do
			local color = options.colors[swatch.setting]
			swatch.tx:SetVertexColor(color.r, color.g, color.b)
			swatch.color = color
		end
		
		GameTooltip:HookScript("OnTooltipSetItem", self.ScanTooltip)
		GameTooltip:HookScript("OnTooltipCleared", function(self)
			self.BoC = nil
		end)

		self:HookAddons()
	end
end

function addon:PLAYER_LOGIN()
	self:RegisterEvent("BAG_UPDATE_DELAYED")
	self:ScanProfessions()
end

function addon:TRADE_SKILL_SHOW()
	newCraftMsgShown = false
end

function addon:TRADE_SKILL_UPDATE()
	if not (IsTradeSkillLinked() or IsTradeSkillGuild()) then
		-- return if not yet fully loaded
		if select(2, GetTradeSkillInfo(1)) ~= "header" then return end
		
		local tradeSkill, skillLevel = GetTradeSkillLine()
		tradeSkill = skillsByName[tradeSkill]

		if tradeSkill then
			tradeSkill = self.player.skills[tradeSkill]

			tradeSkill.link = GetTradeSkillListLink()
			tradeSkill.skillLevel = skillLevel
			local newCraft
			local crafts = tradeSkill.crafts

			for i = 1, GetNumTradeSkills() do
				local recipeLink = GetTradeSkillRecipeLink(i)
				
				if recipeLink then
					local craftID = tonumber(recipeLink:match("enchant:(%d+)"))
					if not crafts[craftID] then
						crafts[craftID] = true
						newCraft = true
					end
				end
			end
			if newCraft then
				self:ClearCache()
			end
		end
	end
end

function addon:LEARNED_SPELL_IN_TAB(spellID)
	if specs[spellID] then
		self:ScanProfessions()
	end
end

function addon:BANKFRAME_OPENED()
	bankOpen = true
	self:ScanInventory()
end

function addon:BANKFRAME_CLOSED()
	bankOpen = false
end

function addon:CHAT_MSG_SKILL(msg)
	local tradeSkillName, newLevel = msg:match(ERR_SKILL_UP_SI)
	local tradeSkill = self.player.skills[skillsByName[tradeSkillName]]

	if tradeSkill and newLevel then
		tradeSkill.skillLevel = tonumber(newLevel)
		tradeSkill.link = select(2, GetSpellLink(tradeSkillName))
		self:ClearCache()
	end
	
	local tradeskill = msg:match(ERR_SKILL_GAINED_S)
	
	if tradeskill and skillsByName[tradeskill] then
		self:ScanProfessions()
		self:ClearCache()
	end
end

local sysMsgPatterns = {
	[ERR_LEARN_RECIPE_S] = function()
		if not newCraftMsgShown then
			newCraftMsgShown = true
			addon:ChatMsg("Please open your tradeskill/craft to update data")
		end
	end,
	[ERR_SPELL_UNLEARNED_S] = function(match)
		local spellID = tonumber(match:match("spell:(%d+)"))
		if skills[spellID] then
			-- this spell ID matches a valid tradeskill, so we'll delete it from the database
			addon.player.skills[spellID] = nil
			addon:ScanProfessions()
			addon:ClearCache()
		else
			-- this spell ID matches a valid tradeskill spec, so we'll delete the spec entry
			local skill = specs[spellID]
			if skill then
				addon.player.skills[skill].spec = nil
				addon:ScanProfessions()
				addon:ClearCache()
			end
		end
	end,
}

function addon:CHAT_MSG_SYSTEM(msg)
	for pattern, func in pairs(sysMsgPatterns) do
		local match = msg:match(pattern)
		if match then
			func(match)
			break
		end
	end
end

--[[

menu = {
	realmNames = {
		-- alphabetically sorted array
		"Azjol-Nerub",
		"Bronzebeard",
		"Chromaggus",
	}
	["Azjol-Nerub"] = {
		charNames = {
			"Armina",
			"Belleza",
			"Cordiana",
		},
		data = {
			["Armina"] = {
				skills = {
					2108,
					3273,
				},
			},
		},
	},
	["Bronzebeard"] = {
	},
	["Chromaggus"] = {
	},
}

]]

function addon:BuildMenu()
	wipe(self.menu)
	-- sorted-by-name array for the LDB dropdown
	local realmNames = {}
	for realmName, realmData in pairs(self.db.data) do
		tinsert(realmNames, realmName)
		local charNames = {}
		local chars = {}
		for charName, charData in pairs(realmData) do
			tinsert(charNames, charName)
			local skills = {}
			for spellID in pairs(charData.skills) do
				tinsert(skills, spellID)
			end
			sort(skills)
			chars[charName] = skills
		end
		sort(charNames)
		self.menu[realmName] = {
			charNames = charNames,
			data = chars,
		}
	end
	sort(realmNames)
	self.menu.realmNames = realmNames
end

function addon:ScanProfessions()
	for i = 1, select("#", GetProfessions()) do
		local prof = select(i, GetProfessions())
		if prof then
			local name, _, skillLevel, _, numAbilities, spellOffset = GetProfessionInfo(prof)
			local skillID = skillsByName[name]
			if skillID then
				local skills = self.player.skills
				skills[skillID] = skills[skillID] or {crafts = {}}
				local _, link = GetSpellLink(skillID)
				-- links are not available on first PLAYER_LOGIN, in which case we wait for UNIT_NAME_UPDATE
				if not link then
					return
				end
				skills[skillID].skillLevel = skillLevel
				skills[skillID].link = link
				if numAbilities == 2 then
					-- we have two profession abilities, one of them may be a profession spec
					skills[skillID].spec = specsByName[GetSpellInfo(spellOffset + 2, BOOKTYPE_PROFESSION)]
				end
			end
		end
	end
	-- if we reached this point, the links were available and we no longer need the following event
	self:UnregisterEvent("UNIT_NAME_UPDATE")
	self:BuildMenu()
end

addon.UNIT_NAME_UPDATE = addon.ScanProfessions

local scanTooltip = CreateFrame("GameTooltip", "BoCScanTooltip", nil, "GameTooltipTemplate")

function addon:GetRecipeInfo(itemLink)
	local skillLevel, specialReq

	scanTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	scanTooltip:SetHyperlink(itemLink)
	local numLines = scanTooltip:NumLines()
	-- parse required skill level and potential profession spec
	for i = 2, numLines do
		local text = _G["BoCScanTooltipTextLeft"..i]:GetText()
		if text then
			skillLevel = text:match(ITEM_MIN_SKILL)
			if skillLevel then
				break
			end
		end
	end
	for i = 2, numLines do
		local text = _G["BoCScanTooltipTextLeft"..i]:GetText()
		if text then
			local req = text:match(ITEM_REQ_SKILL)
			if specsByName[req] then
				specialReq = req
				break
			end
		end
	end
	return tonumber(itemLink:match("item:(%d+)")), skillsByName[self:IsRecipe(itemLink)], tonumber(skillLevel), specialReq
end

function addon:IsRecipe(itemLink)
	if not itemLink then return end
	local type, subType = select(6, GetItemInfo(itemLink))
	return (type == RECIPE and subType ~= BOOK and subType)
end

function addon:ScanBag(bag, src)
	for slot = 1, GetContainerNumSlots(bag) do
		local itemID = GetContainerItemID(bag, slot)
		local _, count, _, _, _, _, link = GetContainerItemInfo(bag, slot)
		if itemID and self:IsRecipe(link) then
			local recipes = self.player.recipes[src]
			recipes[itemID] = (recipes[itemID] or 0) + count
		end
	end
end

function addon:ScanInventory()
	self:ClearCache()
	-- Clear character "InBank" list
	local recipes = self.player.recipes
	recipes.bags = {}
	
	for bag = 0, 4 do
		self:ScanBag(bag, "bags")
	end

	if bankOpen then
		recipes.bank = {}
		self:ScanBag(BANK_CONTAINER, "bank")
		-- Search for recipes in bank bags
		for bag = 5, 11 do
			self:ScanBag(bag, "bank")
		end
	end
end

addon.BAG_UPDATE_DELAYED = addon.ScanInventory

local cache = {
	usable = {},
	lowSkill = {},
	known = {},
	noSpec = {},
	bags = {},
	bank = {},
}

function addon:GetAltsInfo(itemLink)
	local usable, lowSkill, known, noSpec, bags, bank = cache.usable, cache.lowSkill, cache.known, cache.noSpec, cache.bags, cache.bank
	local itemID, tradeSkill, skillLevel, specialReq = self:GetRecipeInfo(itemLink)
	local crafts = craftsDB[tradeSkill]

	for k, v in pairs(cache) do
		wipe(v)
	end

	for charName, v in pairs(self.realm) do
		local recipes = v.recipes
		local inBags = recipes.bags[itemID]
		local inBank = recipes.bank[itemID]
		bags[#bags + 1] = inBags and format("%s (%d)", charName, inBags)
		bank[#bank + 1] = inBank and format("%s (%d)", charName, inBank)
		local skillData = v.skills[tradeSkill]
		if skillData --[[and (options.showPlayer or charName ~= myName)]] and not (v.ignore or skillData.ignore) and (not options.sameFaction or v.faction == myFaction) then
			if skillData.crafts[LCI:GetRecipeLearnedSpell(itemID)] then
				-- if this recipe exists in the character's database, then the character knows it
				known[#known + 1] = charName
			else
				if specialReq and specsByName[specialReq] ~= skillData.spec then
					local text = charName
					if options.showSkillLevel then
						text = format("%s (%d)", text, skillData.skillLevel)
					end
					noSpec[#noSpec + 1] = text
				else
					-- if no spec is required, or the character has that spec, then they'll be able to learn this recipe..
					local text = charName
					if options.showSkillLevel then
						text = format("%s (%d)", text, skillData.skillLevel)
					end
					if skillData.skillLevel >= skillLevel then
						-- ..now, since the character's skill level is high enough..
						usable[#usable + 1] = text
					else
						-- ..or later
						lowSkill[#lowSkill + 1] = text
					end
				end
			end
		end
	end

	return usable, lowSkill, known, noSpec, bags, bank
end

local isUseful

local function dostuff(data, caption, color)
	if #data > 0 then
		tooltip:AddLine(caption, color)
		for i, v in ipairs(data) do
			tooltip:AddLine("  "..v, color)
		end
		isUseful = true
	end
end

local function dostuff2(tooltip, data, caption, color)
	if #data > 0 then
		if color then
			tooltip:AddLine(caption..table.concat(data, ", "), color.r, color.g, color.b)
		else
			tooltip:AddLine(caption..table.concat(data, ", "), 1.0, 1.0, 1.0)
		end
		isUseful = true
	end
end

-- add extra tooltip information if necessary
function addon:ScanTooltip()
	local _, itemLink = self:GetItem()
	if self.BoC or not addon:IsRecipe(itemLink) then
		lastRecipe = nil
		return
	end
	self.BoC = true
	
	local colors = options.colors

	local usable, lowSkill, known, noSpec, bags, bank = cache.usable, cache.lowSkill, cache.known, cache.noSpec, cache.bags, cache.bank

	-- if this recipe is not the same as the last scanned recipe, we need to get new data
	if itemLink ~= lastRecipe then
		usable, lowSkill, known, noSpec, bags, bank = addon:GetAltsInfo(itemLink)
		lastRecipe = itemLink
	end

	isUseful = false
	
	if options.separateTooltip then
		tooltip:SetOwner(self, "ANCHOR_NONE")

		dostuff(usable, "Usable:", colors.usable)
		dostuff(lowSkill, "Insufficient skill:", colors.lowSkill)
		dostuff(noSpec, "Wrong spec:", colors.noSpec)
		dostuff(known, "Already known:", colors.knownBy)

		if not isUseful then
			tooltip:AddLine("Unuseful recipe")
		end

		dostuff(bags, "In bags: ")
		dostuff(bank, "In bank: ")

		addon:TooltipShow(self)
	else
		self:AddLine(" ")

		dostuff2(self, usable,	"Usable: ",				colors.usable)
		dostuff2(self, lowSkill,"Insufficient skill: ",	colors.lowSkill)
		dostuff2(self, noSpec,	"Wrong spec: ",			colors.noSpec)
		dostuff2(self, known,	"Already known: ",		colors.knownBy)

		if not isUseful then
			self:AddLine("Unuseful recipe", 1.0, 1.0, 1.0)
		end

		dostuff2(self, bags, "In bags: ")
		dostuff2(self, bank, "In bank: ")
	end
end

function addon:TooltipShow(frame)
	local tooltipWidth = tooltip:GetWidth()
	local left = (frame:GetLeft() or 0) - tooltipWidth
	local right = GetScreenWidth() - (frame:GetRight() or 0) - tooltipWidth
	if left < right then
		tooltip:SetPoint("TOPLEFT", frame, "TOPRIGHT")
	else
		tooltip:SetPoint("TOPRIGHT", frame, "TOPLEFT")
	end

	local color = TOOLTIP_DEFAULT_BACKGROUND_COLOR
	tooltip:SetBackdropColor(color.r, color.g, color.b)
	tooltip:Show()
end

function addon:DeleteCharacterData(realm, name, skill)
	local data = BookofCraftsDB.data[realm]
	if skill then
		local skills = data[name].skills
		wipe(skills[skill].crafts)
		skills[skill] = nil
		self:ScanProfessions()
	else
		data[name] = nil
	end
	-- self:ChatMsg(realm..name..(skill and skills[skill] or "")..BC_MSG_DATA_DELETED)
	self:ClearCache()
	self:BuildMenu()
end

function addon:ClearCache()
	lastRecipe = nil
	wipe(recipeColorCache)
	for k, v in pairs(self.colorRecipesFunc) do
		if options.colorRecipes[k] then
			v()
		end
	end
end

function addon:GetRecipeColor(itemLink)
	local color = recipeColorCache[itemLink]
	if not color then
		local usable, lowSkill, knownBy, noSpec = self:GetAltsInfo(itemLink)
		color = options.colors.useless
		if #noSpec > 0 then
			color = options.colors.noSpec
		elseif #usable > 0 then
			color = options.colors.usable
			if #knownBy > 0 then
				color = {r = 0, g = 1, b = 1}
			end
		elseif #knownBy > 0 then
			color = options.colors.knownBy
		elseif #lowSkill > 0 then
			color = options.colors.lowSkill
		end
		recipeColorCache[itemLink] = color
	end
	return color
end

-- function addon:ColorRecipes(loc)
	-- if options.colorRecipes[loc] then
		-- colorRecipesFunc[loc]()
	-- end
-- end

local function colorBagRecipes(self)
	if not self.size then
		return
	end
	local id = self:GetID()
	local bagName = self:GetName()
	for i = 1, self.size do
		local button = _G[bagName.."Item"..i]
		local slot = button:GetID()
		local link = GetContainerItemLink(id, slot)
		if options.colorRecipes.bags and link and addon:IsRecipe(link) then
			local color = addon:GetRecipeColor(link)
			SetItemButtonTextureVertexColor(button, color.r, color.g, color.b)
		else
			SetItemButtonTextureVertexColor(button, 1.0, 1.0, 1.0)
		end
	end
end

local function colorBankRecipes(self)
	local link = GetInventoryItemLink("player", self:GetInventorySlot())
	if options.colorRecipes.bank and link and addon:IsRecipe(link) then
		local color = addon:GetRecipeColor(link)
		SetItemButtonTextureVertexColor(self, color.r, color.g, color.b)
	else
		SetItemButtonTextureVertexColor(self, 1.0, 1.0, 1.0)
	end
end

local function colorGuildBankRecipes()
	if GuildBankFrame.mode == "bank" then
		local tab = GetCurrentGuildBankTab()
		for i = 1, MAX_GUILDBANK_SLOTS_PER_TAB do
			local index = mod(i, NUM_SLOTS_PER_GUILDBANK_GROUP)
			if index == 0 then
				index = NUM_SLOTS_PER_GUILDBANK_GROUP
			end
			local column = ceil((i - 0.5) / NUM_SLOTS_PER_GUILDBANK_GROUP)
			local button = _G["GuildBankColumn"..column.."Button"..index]
			local link = GetGuildBankItemLink(tab, i)
			if options.colorRecipes.guildBank and link and addon:IsRecipe(link) then
				local color = addon:GetRecipeColor(link)
				SetItemButtonTextureVertexColor(button, color.r, color.g, color.b)
			else
				SetItemButtonTextureVertexColor(button, 1.0, 1.0, 1.0)
			end
		end
	end
end

local function colorVendorRecipes()
	if not options.colorRecipes.vendor then return end
	for i = 1, min(MERCHANT_ITEMS_PER_PAGE, GetMerchantNumItems()) do
		local index = ((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE) + i
		local link = GetMerchantItemLink(index)
		if not link then return end
		if addon:IsRecipe(link) then
			local _, _, _, _, numAvailable, isUsable = GetMerchantItemInfo(index)
			local color = addon:GetRecipeColor(link)
			local r, g, b = color.r, color.g, color.b
			local merchantButton = _G["MerchantItem"..i]
			local itemButton = _G["MerchantItem"..i.."ItemButton"]
			-- SetItemButtonNameFrameVertexColor(merchantButton, r, g, b)
			if numAvailable == 0 then
				r, g, b = r * 0.5, g * 0.5, b * 0.5
				SetItemButtonSlotVertexColor(merchantButton, r, g, b)
				SetItemButtonTextureVertexColor(itemButton, r, g, b)
				SetItemButtonNormalTextureVertexColor(itemButton, r, g, b)
			else
				SetItemButtonSlotVertexColor(merchantButton, r, g, b)
				SetItemButtonTextureVertexColor(itemButton, r, g, b)
				SetItemButtonNormalTextureVertexColor(itemButton, r, g, b)
			end
		end
	end
end

local function colorVendorBuybackRecipes()
	if not options.colorRecipes.vendor then return end
	for i = 1, min(MERCHANT_ITEMS_PER_PAGE, GetNumBuybackItems()) do
		local link = GetBuybackItemLink(i)
		if not link then return end
		if addon:IsRecipe(link) then
			local color = addon:GetRecipeColor(link)
			local r, g, b = color.r, color.g, color.b
			local merchantButton = _G["MerchantItem"..i]
			local itemButton = _G["MerchantItem"..i.."ItemButton"]
			SetItemButtonNameFrameVertexColor(merchantButton, r, g, b)
			SetItemButtonSlotVertexColor(merchantButton, r, g, b)
			SetItemButtonTextureVertexColor(itemButton, r, g, b)
			SetItemButtonNormalTextureVertexColor(itemButton, r, g, b)
		end
	end
end

local function colorAuctionRecipes()
	if not options.colorRecipes.auction then return end
	for i = 1, min(NUM_BROWSE_TO_DISPLAY, GetNumAuctionItems("list")) do
		local itemLink = GetAuctionItemLink("list", i + FauxScrollFrame_GetOffset(BrowseScrollFrame))
		if itemLink and addon:IsRecipe(itemLink) then
			local texture = _G["BrowseButton"..i.."ItemIconTexture"]
			if not texture then
				break
			end
			local color = addon:GetRecipeColor(itemLink)
			texture:SetVertexColor(color.r, color.g, color.b)
		end
	end
end

-- these functions are called whenever the corresponding setting is toggled
local colorRecipesFunc = {
	bags = function()
		for i = 1, NUM_CONTAINER_FRAMES do
			colorBagRecipes(_G["ContainerFrame"..i])
		end
	end,
	bank = function()
		for i = 1, NUM_BANKGENERIC_SLOTS do
			BankFrameItemButton_Update(_G["BankFrameItem"..i])
		end
	end,
	guildBank = function()
		if addonLoaded.Blizzard_GuildBankUI and GuildBankFrame:IsShown() then
			GuildBankFrame_Update()
		end
	end,
	vendor = function()
		if MerchantFrame:IsShown() then
			MerchantFrame_UpdateMerchantInfo()
			-- MerchantFrame_UpdateBuybackInfo()
		end
	end,
	auction = function()
		if addonLoaded.Blizzard_AuctionUI and AuctionFrame:IsShown() then
			AuctionFrameBrowse_Update()
		end
	end,
}
addon.colorRecipesFunc = colorRecipesFunc

hooksecurefunc("ContainerFrame_Update", colorBagRecipes)
hooksecurefunc("ContainerFrame_UpdateCooldown", function(container, button)
	local link = GetContainerItemLink(container, button:GetID())
	if options.colorRecipes.bags and link and addon:IsRecipe(link) then
		local color = addon:GetRecipeColor(link)
		SetItemButtonTextureVertexColor(button, color.r, color.g, color.b)
	end
end)
hooksecurefunc("BankFrameItemButton_Update", colorBankRecipes)
hooksecurefunc("MerchantFrame_UpdateMerchantInfo", colorVendorRecipes)
hooksecurefunc("MerchantFrame_UpdateBuybackInfo", colorVendorBuybackRecipes)


local addonHooks = {
	Blizzard_AuctionUI = "AuctionFrameBrowse_Update",
	Blizzard_GuildBankUI = "GuildBankFrame_Update",
}

local hookFuncs = {
	AuctionFrameBrowse_Update = colorAuctionRecipes,
	GuildBankFrame_Update = colorGuildBankRecipes,
}

function addon:HookAddon(addon)
	local hook = addonHooks[addon]
	if hook then
		hooksecurefunc(hook, hookFuncs[hook])
		addonLoaded[addon] = true
		addonHooks[addon] = nil
		if not next(addonHooks) then
			self:UnregisterEvent("ADDON_LOADED")
			self.ADDON_LOADED = nil
		end
		return true
	end
end


function addon:HookAddons()
	for addon in pairs(addonHooks) do
		if IsAddOnLoaded(addon) then
			self:HookAddon(addon)
		end
	end
end